#!/bin/bash
#
# v0.1  2015/10/24
#
# solar-hack.sh - Imports SMA Solar Inverter data to Indigo Domotics 6 for Mac
#
# Note: You must have SMASPot running already - see https://sbfspot.codeplex.com/
#
# Licensed under GNU GPL v3 license 2015 jtodd@loligo.com
#
#
# This is a horrendously hackish method for updating Indigo Domotics v6
#  servers with variables collected from SMA Solar Inverters. There
#  is a program called SMASpot which talks to the inverters via UDP
#  RPC calls, and then dumps the results into both a database and a
#  CSV file.  I will have to assume you've got that set up.
#  What this shell script does is then extract the values from CSV and create
#  a Python file, which is then executed by calling the Indigo IPH.
#  Why this way?  Because I don't know Python, and I didn't feel like
#  learning it for this task. :-)  I'm only using the CSV files.
#
# This script should be put into cron to run every five minutes, probably
#  on the minute after you expect your SMASpot to run.  I chose a strategy
#  like this:
#
#  [/tmp] apple% crontab -l
#  1,6,12,18,24,32,38,44,50,56 * * * * /usr/local/bin/solar-hack > /dev/null 2>&1
#  [/tmp] apple%
#
# Of course, ensure your clocks are synchronized with NTP.
#
# Alternately,  you could call the script from the "Schedule" function
#  within Indigo so you wouldn't have to mess with cron.
#
#
# Indigo is kind of fussy about variables, and they need to be named
#  before you can modify them. YOU WILL NEED TO SET UP THE VARIABLES
#  THE FIRST TIME YOU RUN THIS PROGRAM.  By adding a suffix of "-s"
#  to the script, it will iterate through the values and create them
#  in Indigo.  This only needs to be done once, and if you try to run
#  it again it will fail. (Hint: if you add or change something, just
#  delete all the variables from Indigo's "variable" panel and start
#  from scratch, or manually add each one if you want to keep your
#  variable mappings in place.)  Running with "-s" also creates a
#  file in /tmp that is suitable for manual importation to IndigoPlotD
#  so that you don't have to tediously add all the values by hand if
#  you run that package.
#
# I've already set up SBFspot on a Raspberry Pi running elsewhere
#  on my network.  I suppose you could get it to work on the Mac,
#  which would be more efficient.  These first few steps are just
#  to copy the files from the remote machine to my local Mac; you
#  might be able to modify/delete them if you're running SBFspot
#  locally.
#
#
# Where are the data files for SMASpot?
datadir=/usr/local/var/smadata
# What's your plant name?
plantname=MySolarPlantHere
# Serial numbers of your inverters - if you have only one, just put in one.
listofinverters="1913111234 1913115678 19131290123"
# How many inverters do you have?
numberofinverters=3

# Sync the CSV (and other) files from my remote Raspberry Pi running SMASpot.
rsync -arx -e "ssh -l pi -i /Users/apple/.ssh/root-id_rsa" 192.168.1.161:/home/pi/smadata/ $datadir


# first, set up some values for later use
today=`date "+%Y%m%d"`
year=`date "+%Y"`
filename="$datadir/$year/$plantname-Spot-$today.csv"
outfile="$datadir/solarupdate.py"
indigoplotdfile="/tmp/indigoplotd.py"

# Get rid of the old Python file that was used during the last update
if [ -f $outfile ]; then
  rm $outfile
fi

# Get rid of the old IndigoPlotD file, if it exists
if [ -f $indigoplotdfile ]; then
  rm $indigoplotdfile ;
fi

function updatevariables {
# let's define the array inverter serial numbers as a list
#  so we can run through them one at a time. You will need to
#  change this to your list of inverters, or just a single
#  serial number if you only have one inverter.  Find the
#  serial number in the fourth column of the .csv file output
for inverter in `echo $listofinverters`
 do
  # Give every variable a name that is unique by prefixing the last three digits of serial number
  #  If you by some chance have two inverters with the same three ending digits, use the last four or five digits
  shortname=`echo $inverter|tail -c -4`
  invertervars=`tail -$numberofinverters $filename|grep $inverter`
  i=1
  # Now, define all of the values stored in each CSV line (it's a long list, sorry)
  for solarvar in Timestamp DeviceName DeviceType Serial Pdc1 Pdc2 Idc1 Idc2 Udc1 Udc2 Pac1 Pac2 Pac3 Iac1 Iac2 Iac3 Uac1 Uac2 Uac3 PdcTot PacTot Efficiency EToday ETotal Frequency OperatingTime FeedInTime BTSignal Condition GridRelay Temperature
   do
    loopvariable=`echo $invertervars|cut -d ";" -f $i`
    finalname="SMA$shortname$solarvar"
    # This is the Indigo Python command to change the variable - put it at the end of the file...
    echo "indigo.variable.updateValue(\"$finalname\", \"$loopvariable\")" >> $outfile
    ((i++))
   done
 done
}


# This function only runs when you use "-s" modifier with the script.
# It creates two files: one that tells Indigo about all of the new
# variables to create, and one that creates a python file that can
# be used by IndigoPlotD as a quick method of telling that plugin to
# start tracking all of those new variables through history, and
# the details about those variables.
#
function firsttime {
 for inverter in `echo $listofinverters`
  do
    shortname=`echo $inverter|tail -c -4`
    invertervars=`tail -$numberofinverters $filename|grep $inverter`
    i=1
    for solarvar in Timestamp DeviceName DeviceType Serial Pdc1 Pdc2 Idc1 Idc2 Udc1 Udc2 Pac1 Pac2 Pac3 Iac1 Iac2 Iac3 Uac1 Uac2 Uac3 PdcTot PacTot Efficiency EToday ETotal Frequency OperatingTime FeedInTime BTSignal Condition GridRelay Temperature
     do
      loopvariable=`echo $invertervars|cut -d ";" -f $i`
      finalname="SMA$shortname$solarvar"

      # This creates a new line in the Python file that will be run at the end of
      #  this script. Each line will create a new variable in Indigo to store
      #  the results from the SMASpot CSV file when it's parsed.
      echo "indigo.variable.create(\"$finalname\", \"\")" >> $outfile


      # IndigoPlotD can only plot stuff that is numeric, so any non-integer data
      #  we don't pay attention to.  If you want to track things like GridStatus
      #  you will have to use a trigger or schedule in Indigo and create a
      #  different integer variable to store that data, but that's outside the scope
      #  of this script.
      #
      # NOTE: script assumes a 100000 maximum value for any of these tracked figures
      #
      for item in Pdc1 Pdc2 Idc1 Idc2 Udc1 Udc2 Pac1 Pac2 Pac3 Iac1 Iac2 Iac3 Uac1 Uac2 Uac3 PdcTot PacTot Efficiency EToday ETotal Frequency OperatingTime FeedInTime BT_Signal Temperature
       do
         if [ "$solarvar" == "$item" ]; then
            writeindigod
         fi
       done

      # Add one to the loop counter so we know what field to cut, and then loop
      ((i++))
     done
  done
}



# Also when we run with the "-s" option, let's also create a file in /tmp
#  that can then be imported into IndigoPlotD (an extension for Indigo)
#  so that all the values start getting tracked by that system.  You will
#  need to copy the file from /tmp and put it into IndigPlotD's directory:
#    cp /tmp/indigoplotd.py ~/Documents/INDIGOplotD/py
#  and then go to the IndigoPlotD plugin menu in Indigo and "Select file to restore"
#  to import this particular file and start tracking the values.
#  If you don't run IndigoPlotD you can ignore this file and it will
#  be deleted on reboot since it's in /tmp

function writeindigod {
  echo "
ppp ={\"deviceOrVariableName\":u'$finalname'
     ,\"devOrVar\":         u'Var-'
     ,\"state\":            u'value'
     ,\"measurement\":      u'average'
     ,\"multiplier\":       u'1.0'
     ,\"offset\":           u'0.0'
     ,\"minValue\":         u'0.0'
     ,\"maxValue\":         u'100000'
     ,\"fillGaps\":         u'0'
     ,\"resetType\":        u'\"\"'
      ,\"nickName\":         u'\"$finalname\"'
      ,\"logLevel\":         u'1'
      }
plug.executeAction(\"addDeviceAndStateToSelectionList\" , props =ppp) " >> $indigoplotdfile
   }

function sendtoindigo {
# Now, we execute the Python file we created with all the variable commands in it
/Library/Application\ Support/Perceptive\ Automation/Indigo\ 6/IndigoPluginHost.app/Contents/MacOS/IndigoPluginHost -x $outfile
}


# Main processing
#
if [ "$1" = "-s" ]; then firsttime
 else updatevariables
 fi
sendtoindigo
exit
